<?php

App::uses('AppComponent', 'Controller/Component');
App::uses('OrderStatus', 'EvCheckout.Model');
App::uses('BasketLib', 'EvBasket.Lib');
App::uses('RouterUtil', 'Routable.Lib');

class OrderManagerComponent extends AppComponent {

	public $order = array();

	public $orderItems = array();

	public $orderTotals = array();

	/**
	 * create order totals row
	 *
	 * @param 	string 	$name 		The name of the total (label)
	 * @param 	float 	$amount 	The amount of the total
	 * @param 	int 	$sequence 	The sequence order number
	 * @return 	bool
	 */
	public function setupOrderTotal($name, $amount, $sequence = 10) {
		$this->orderTotals[$name] = array(
			'name' => $name,
			'amount' => $amount,
			'sequence' => $sequence
		);

		return true;
	}

	/**
	 * create order item row
	 *
	 * @param 	string 			$name 			Name string or array from basket of model / model_id
	 * @param 	int 			$quantity 		Number of items
	 * @param 	float 			$unitPrice 		Unit Price of the item
	 * @param 	float 			$rowTotal 		Row total (unitprice X quantity)
	 * @param 	float 			$taxRate 		The tax rate (percentage) to apply
	 * @param 	array 			$orderItemData 	Array of any extra data to store
	 * @return 	bool
	 */
	public function setupOrderItem($name, $quantity, $unitPrice, $rowTotal, $taxRate, $orderItemData = array()) {
		$this->orderItems[] = array(
			'name' => $name,
			'quantity' => $quantity,
			'unit_price' => $unitPrice,
			'row_total' => $rowTotal,
			'tax_rate' => $taxRate,
			'OrderItemData' => $orderItemData
		);

		return true;
	}

	/**
	 * generate the order item data array
	 *
	 * @param 	array 	$itemDataPaths 	The array of data from the config to searhc for
	 * @param 	array 	$item 			The array of data to search in (usual basketItemRow)
	 * @return 	array 	$orderItemData 	Correct format for storing in the database
	 */
	public function setupOrderItemData($itemDataPaths, $item) {
		foreach ($itemDataPaths as $name => $path) {
			$orderItemData[] = array(
				'name' => $name,
				'item_data' => $this->findItemData($item, $path)
			);
		}

		return $orderItemData;
	}

	/**
	 * given the array of data and the path - extract the data we want to save
	 *
	 * @param 	array 	$itemData 	Item data to search in
	 * @param 	string 	$path 		The path to search data for
	 * @return  mixed|bool
	 */
	public function findItemData($itemData, $path) {
		return Hash::get($itemData, $path, false);
	}

	/**
	 * create main order details
	 *
	 * @param 	array 	$user 			Array of user data
	 * @param 	array 	$addresses 		Array of address data
	 * @param 	int 	$orderStatusId 	Id of the order status to set
	 * @param 	bool 	$isPaid 		Whether the order has been paid or not
	 * @return  bool
	 */
	public function setupOrder($user, $addresses, $orderStatusId = OrderStatus::UNPAID, $isPaid = false) {
		$this->order = array(
			'name' => (! empty($user['name'])) ? $user['name'] : '',
			'email' => (! empty($user['email'])) ? $user['email'] : '',
			'phone' => (! empty($user['phone'])) ? $user['phone'] : '',
			'billing_address' => (! empty($addresses['billing'])) ? $addresses['billing'] : '',
			'delivery_address' => (! empty($addresses['delivery'])) ? $addresses['delivery'] : '',
			'order_status_id' => $orderStatusId,
			'is_paid' => $isPaid,
			'is_guest_checkout' => (isset($user['is_guest_user'])) ? $user['is_guest_user'] : '1'
		);

		return true;
	}

	/**
	 * save the order we've built up
	 *
	 * @param 	object 		$Order 		Order model object
	 * @return 	int|bool 	$orderId 	Order ID or false on fail
	 */
	public function saveOrder($Order) {
		$result = $Order->saveAll(
			array(
				'Order' => $this->order,
				'OrderItem' => $this->orderItems,
				'OrderTotal' => $this->orderTotals
			),
			array(
				'deep' => true
			)
		);

		if ($result === true) {
			return $Order->id;
		}

		return false;
	}

	/**
	 * create and setup an order from a basket
	 *
	 * @param 	array 		$basket 	Array of Basket Data
	 * @param 	array 		$user 		array of user details
	 * @param 	array 		$addresses 	array of address details
	 * @return 	int|bool 	$orderId 	Return the order ID number
	 */
	public function createFromBasket($basket, $user, $addresses) {
		$this->setupOrder($user, $addresses);

		$orderItemData = Configure::read('EvCheckout.orderItemData');

		foreach ($basket['BasketItem'] as $item) {
			$itemData = array();

			if (isset($orderItemData[$item['model']])) {
				$itemData = $this->setupOrderItemData($orderItemData[$item['model']], $item);
			}

			$this->setupOrderItem(
				BasketLib::getItemName($item),
				$item['quantity'],
				$item['unit_price'],
				$item['row_total'],
				$item['tax_rate'],
				$itemData
			);
		}

		foreach ($basket['BasketTotal'] as $total) {
			$this->setupOrderTotal(
				$total['name'],
				$total['amount'],
				$total['sequence']
			);
		}

		$orderId = $this->saveOrder(EvClassRegistry::init('EvCheckout.Order'));

		if ($orderId !== false) {
			if (! empty($basket['Basket']['id'])) {
				$Basket = EvClassRegistry::init('EvBasket.Basket');
				$Basket->delete($basket['Basket']['id']);
			}

			return $orderId;
		}

		return false;
	}

	/**
	 * return the order complete url
	 *
	 * @return string 	$orderCompleteUrl
	 */
	public function getCompleteUrl() {
		return Router::url(
			RouterUtil::getListingRoute('EvCheckout', 'Order', 'complete'),
			true
		);
	}

	/**
	 * get the order cancelled url
	 *
	 * @return string 	$orderCancelledUrl
	 */
	public function getCancelledUrl() {
		return Router::url(
			RouterUtil::getListingRoute('EvCheckout', 'Order', 'cancel'),
			true
		);
	}

	/**
	 * build the order from the given transactionId
	 *
	 * @param 	int 	$transactionId 		The successful transaction ID just completed
	 * @return 	int 	$orderId 			The order we've just built
	 */
	public function buildOrder($transactionId) {
		$selectedBuilder = Configure::read('EvCheckout.orderBuilder');
		list($plugin, $builder) = pluginSplit($selectedBuilder, true, '');

		App::uses($builder, $plugin . 'OrderBuilders');

		$OrderBuilder = new $builder($this->_controller);

		return $OrderBuilder->build($transactionId);
	}

}
